/**************************************************************************/
/* FILE NAME: main.c                       COPYRIGHT (c) Freescale 2009   */
/*                                                All Rights Reserved     */
/* DESCRIPTION:                                                           */
/* Demo code to test functionality of Spectrum Cluster Board                         */
/*                                                                        */
/* SETUP:                                                                 */
/* Software tested on Spectrum Cluster Board using Lauterbach and CodeWarrior compiler  */
/*                                                                        */
/*========================================================================*/
/* REV      AUTHOR        DATE        DESCRIPTION OF CHANGE               */
/* ---   -----------    ----------    ---------------------               */
/* 0.1    D. Mckenna       22/Apr/09    Initial Version                     */
/**************************************************************************/
#include "MPC5606S_2.h"
#include "Ports.h"


/*========================================================================*/
/*                          DEFINES                                       */
/*========================================================================*/
#define MPH_DIAL_LED PC12
#define FUEL_DIAL_LED PC14
#define TEMP_DIAL_LED PC15
#define RPM_DIAL_LED PC13
#define FUEL_LED PJ14
#define TEMP_LED PK1
#define ENGINE_LED PK0
#define OIL_LED PJ15
#define LEFT_LED PK10
#define RIGHT_LED PK11
#define FOG_LED PB11
#define HIBEAM_LED PB10

/*========================================================================*/
/*                      GLOBAL VARIABLES                                  */
/*========================================================================*/
int8_t direction; // can be -1 or 1 - bck or fwd
uint8_t ministep; //current number of ministeps per 90o rotation
uint8_t fullstep;// can be 0-3 - each represents one "full step" e.g. 90o rotation
uint32_t count; /* tally for the number of steps taken */
uint16_t new_pos; /* posotion motor should move to */
uint16_t current_pos; /* current motor position */
uint16_t accel_value, decel_value; /* position at which accel/decel should occur */
uint8_t move_complete; /* flag so that main cde knows when motor has reached desired position */
uint32_t freq; /* clock frequency */
/*========================================================================*/
/*                      INPUTS PARAMETERS                                 */
/*========================================================================*/

/*========================================================================*/
/*                      OUTPUT PARAMETERS                                 */
/*========================================================================*/

/*========================================================================*/
/*                          PROTOTYPES                                    */
/*========================================================================*/
void MC_MODE_INIT();
void DISABLE_WATCHDOG();
uint16_t SetupPLL(uint16_t);
void CONFIG_LEDS(void);
void LIGHT_LEDS(void);
void CONFIG_EMIOS_PWM(void);
void CONFIG_SGL(void);
void CONFIG_SMC(void);
void PIT0_INIT(vint32_t);
void LQ043_setup(uint32_t);
void DCU_init(uint32_t);
void PIT_CH0_ISR(void);

/*========================================================================*/
/*                          FUNCTIONS                                     */
/*========================================================================*/

int main(void)
{
	int32_t i;
	
	DISABLE_WATCHDOG();

	MC_MODE_INIT();   // select 16MHz IRC

	SetupPLL(64);
	freq = 64;
	
	// Configure the interrupts and interrupt controller (done during init code)
	//xcptn_xmpl();     

	// Enable Interrupts at Core (done during init)
	//enableIrq();      

	// CONFIG INTC
	//INTC.PSR[59].R  = 0x02;  /* PIT 0 */
	//Install interrupt handler for PIT0
	INTC_InstallINTCInterruptHandler(PIT_CH0_ISR,59,2);	//Setup priority
	
	/* initialise all LED pins */
	CONFIG_LEDS();
	/* light all LEDs */
	LIGHT_LEDS();
	/* setup PWM for Sound generation */
	CONFIG_EMIOS_PWM();
	/* configure Sound Generation Logic */
	CONFIG_SGL();
	
	/* initialise DCU */
	DCU.DCUMODE.B.RASTER_EN = 1;	//Enable Raster signals
    DCU.DCUMODE.B.DCUMODE = 1; // Standard Mode
	DCU_init(freq);
	
	/* Initialise the Stepper Motor Contoller */
	CONFIG_SMC();
	
	// Enable Interrupts at Core
	INTC.CPR.B.PRI = 0;
	
	while(1)
	{
		/* alter background colour on screen*/
		for(i=0xFF;i>0;i=i-3)
		{
			while(DCU.INTSTATUS.B.VSYNC==0);
			DCU.INTSTATUS.R &= 0x00000001;
			DCU.BGND.B.BGND_R = i;	//Change Red value
			DCU.BGND.B.BGND_G = 0xFF-i;	//Change Green value
		}
		for(i=0xFF;i>0;i=i-3)
		{
			while(DCU.INTSTATUS.B.VSYNC==0);
			DCU.INTSTATUS.R &= 0x00000001;
			DCU.BGND.B.BGND_G = i;	//Change Green value
			DCU.BGND.B.BGND_B = 0xFF-i;	//Change Blue value
		}
		for(i=0xFF;i>0;i=i-3)
		{
			while(DCU.INTSTATUS.B.VSYNC==0);
			DCU.INTSTATUS.R &= 0x00000001;
			DCU.BGND.B.BGND_B = i;	//Change Blue value
			DCU.BGND.B.BGND_R = 0xFF-i;	//Change Red value
		}
	};
} /* main */


void MC_MODE_INIT()
{
	/* Enbable all peripheral clocks */
	CGM.SCDC.R = 0x80808080;
	/* Setting RUN Configuration Register ME_RUN_PC[0] */
	ME.RUNPC[0].R=0x000000FE; /* Peripheral ON in every mode */
	CGM.OCDSSC.B.SELCTL=1; 	   	   //Select Fast IRC (16 MHz)

	/* Re-enter in DRUN mode to update */
	ME.MCTL.R = 0x30005AF0;     	  /* Mode & Key */
	ME.MCTL.R = 0x3000A50F;     	  /* Mode & Key */  
}                                                                                                                                                                                                                         

void DISABLE_WATCHDOG()
{
	SWT.SR.R = 0x0000c520; /* key */
	SWT.SR.R = 0x0000d928; /* key */
	SWT.CR.R = 0x8000010A; /* disable WEN */
}                                                                                                                                                                                                                                                                   

//Initial PLL to freqMHz and select as system bus - assume 8MHz crystal
//freq must be >= 16 and <= 64
//Returns value actually set
uint16_t SetupPLL(uint16_t freq)
{
    if (freq > 64) freq = 64;
    if (freq < 16) freq = 16;
    ME.DRUN.B.XOSC0ON=1;			       /* Switch on external osc */
    ME.DRUN.B.SYSCLK=2;			       /* Select external osc */
/* RE enter the drun mode, to update the configuration */
    ME.MCTL.R = 0x30005AF0;        /* Mode & Key */
    ME.MCTL.R = 0x3000A50F;        /* Mode & Key inverted */
    while(ME.GS.B.S_OSC==0);			   /* Wait for external osc to stabilize */
    while(ME.GS.B.S_MTRANS==1);      /* Wait for mode entry to complete */
    while(ME.GS.B.S_CURRENTMODE!=3); /* Check DRUN mode has been entered */
	
/* Max bus 64Mhz, VCO 256-512Mhz */
/* Fpll = XTAL * NDIV / IDF / ODF = 64MHz */
/* Fvco = XTAL * NDIV / IDF = 512MHz */
    if (freq >= 32)
    {
	CGM.FMPLL[0].CR.B.IDF=0x0000; 	/* Divide by 1 */
	CGM.FMPLL[0].CR.B.ODF=0x2;	/* Divide by 8 */
	CGM.FMPLL[0].CR.B.NDIV=freq;	/* Loop divide by freq */
    }
    else if (freq >= 16)
    {
	CGM.FMPLL[0].CR.B.IDF=0x0000;	/* Divide by 1 */
	CGM.FMPLL[0].CR.B.ODF=0x3;	/* Divide by 16 */
	CGM.FMPLL[0].CR.B.NDIV=freq*2;	/* Loop divide by freq*2 */
    }

//    CGM.FMPLL[1].CR.B.EN_PLL_SW	= 1;	//Enable progressive PLL switching
	
    ME.DRUN.B.PLL1ON=1;     /* enable pll */      
      
/* RE enter the drun mode, to update the configuration */
    ME.MCTL.R = 0x30005AF0;     	  /* Mode & Key */
    ME.MCTL.R = 0x3000A50F;         /* Mode & Key inverted */
    while(ME.GS.B.S_MTRANS==1);		    /* Wait for mode entry to complete */
    while(ME.GS.B.S_CURRENTMODE!=0x3);/* Check DRUN mode has been entered */
		
    while(CGM.FMPLL[0].CR.B.S_LOCK==0);
	
    ME.DRUN.B.SYSCLK=0x4;  /* system clock is PLL */
    ME.MCTL.R = 0x30005AF0;     	  /* Mode & Key */
    ME.MCTL.R = 0x3000A50F;         /* Mode & Key inverted */
    while(ME.GS.B.S_MTRANS==1);		    /* Wait for mode entry to complete */
    while(ME.GS.B.S_CURRENTMODE!=0x3);/* Check DRUN mode has been entered */

/* ME_GS Poll Global status register to get current system clock
   fail if system clock is not pll
   0000 16MHz internal RC oscillator
   0001 divided 16MHz internal RC oscillator
   0010 4MHz crystal oscillator
   0011 divided 4MHz crystal oscillator
   0100 system PLL
   1111 system clock is disabled */

    while(ME.GS.B.S_SYSCLK != 4){}; /* fail is stuck here	 */
    return freq;
}

void CONFIG_LEDS(void)
{
	/* this function initialises the pins for all LEDS */
	SIU.PCR[MPH_DIAL_LED ].R = 0x0200;
	SIU.PCR[FUEL_DIAL_LED].R = 0x0200;
	SIU.PCR[TEMP_DIAL_LED].R = 0x0200;
	SIU.PCR[RPM_DIAL_LED].R = 0x0200;
	SIU.PCR[FUEL_LED].R = 0x0200;
	SIU.PCR[TEMP_LED].R = 0x0200;
	SIU.PCR[ENGINE_LED].R = 0x0200;
	SIU.PCR[OIL_LED].R = 0x0200;
	SIU.PCR[LEFT_LED].R = 0x0200;
	SIU.PCR[RIGHT_LED].R = 0x0200;
	SIU.PCR[FOG_LED].R = 0x0200;
	SIU.PCR[HIBEAM_LED].R = 0x0200;
}

void LIGHT_LEDS(void)
{
	/* this function lights all LEDs on the board */
	SIU.GPDO[MPH_DIAL_LED ].R = 1;
	SIU.GPDO[FUEL_DIAL_LED].R = 1;
	SIU.GPDO[TEMP_DIAL_LED].R = 1;
	SIU.GPDO[RPM_DIAL_LED].R = 1;
	SIU.GPDO[FUEL_LED].R = 1;
	SIU.GPDO[TEMP_LED].R = 1;
	SIU.GPDO[ENGINE_LED].R = 1;
	SIU.GPDO[OIL_LED].R = 1;
	SIU.GPDO[LEFT_LED].R = 1;
	SIU.GPDO[RIGHT_LED].R = 1;
	SIU.GPDO[FOG_LED].R = 1;
	SIU.GPDO[HIBEAM_LED].R = 1;
}

void CONFIG_EMIOS_PWM(void)
{  
   /* this function configures eMIOS A channels 16 and 23 as PWMs for use by SGL Module */

   /* set the eMIOS clock source as XOSC */
   CGM.AC1SC.B.SELCTL = 0;
   CGM.AC2SC.B.SELCTL = 0;
   //Configure MCR register
   EMIOS_0.MCR.R         = 0xB4000000;
   EMIOS_0.UCDIS.R       = 0xFF000000;
   // configurchannel 23 of emios0
   EMIOS_0.CH[23].CADR.R = 0x00000400;
   EMIOS_0.CH[23].CBDR.R = 0x00000800;
   EMIOS_0.CH[23].CCR.R  = 0x020602D8;
   //configure channel 16 of emios0
   EMIOS_0.CH[16].CADR.R = 0x00000400;
   EMIOS_0.CH[16].CBDR.R = 0x00000800;
   EMIOS_0.CH[16].CCR.R  = 0x820602D8;
}

void CONFIG_SGL(void)
{
	/* this function configure the Sound Generaltion Logic Module */
	
	// Config the sgl output on PC10
	SIU.PCR[PC10].R = 0x0A0C;
	
	SGL.MODE_SEL.B.ch2_sel=0;
	SGL.MODE_SEL.B.ch1_sel=7;
	SGL.MODE_SEL.B.pre=0;
	SGL.MODE_SEL.B.MP=0; // creat poly sound
	SGL.MODE_SEL.B.SN_CTRL=2; // continuous sound
}

void CONFIG_SMC(void)
{
	/* this function sets up stepper motor pins for use with SSD module */
	uint32_t i;
	vuint32_t sample;
	
	/* initialise all global variables */
	fullstep = 0;
	direction = 1; /* start with motor turning fwds */
	ministep = sample = count = move_complete = 0;
	accel_value = decel_value = 0xffff;
	current_pos = 0xfffe;
	/* setup pins for use with SMC module - using Motor 0 - Motor 3 */
	for(i=PD0;i<PE0;i++)
		SIU.PCR[i].R = 0x600;
	
	SMC.CTL0.B.MCPRE = 2;	//Divide incoming clock by 4
	SMC.CTL1.B.RECIRC = 1;	//Recirculation on low side
	SMC.PER.R = 160;	//Period is 160 clocks
	
	for(i=0;i<8;i=i+2)
	{
		SMC.CC[i].B.MCOM = 3;	//Dual-Full H-Bridge mode
		SMC.CC[i].B.MCAM = 1;	//Left aligned
		SMC.DC[i].R = 0x80A0;	//Different PWM duties
	
		SMC.CC[i+1].B.MCOM = 3;	//Dual-Full H-Bridge mode
		SMC.CC[i+1].B.MCAM = 1;	//Left aligned
		SMC.DC[i+1].R = 0x8000;	
	}
	
	/* ensures motor is a zero upon startup */
	direction = 1;
	PIT0_INIT(0x1fff);		//Initialise PIT interrupt - value sets speed
}

void PIT0_INIT(vint32_t LDVAL)
{
	PIT.CH[0].LDVAL.R         = LDVAL;  /* load PIT counter */
	PIT.CH[0].TCTRL.B.TIE     = 1;      /* enable interrupt */
	PIT.CH[0].TCTRL.B.TEN     = 1;      /* enable channel */
	PIT.PITMCR.B.MDIS         = 0;      /* enable PIT module */
}



void PIT_CH0_ISR(void)
{
	/* this function contorls the signals used to drive the stepper motors */
	vuint16_t i,sign;
	
	PIT.CH[0].TFLG.B.TIF    = 1;  /* clear interrupt flag */

	switch(fullstep)
	{
	case 0:
		if(ministep==15&&direction==1)
		{
			fullstep++; // next fullstep
			ministep = 0; // reset ministep count
			current_pos++;
			for(i=0;i<8;i=i+2)
			{
				SMC.DC[i].R = 0x0000;	//turn off
				SMC.DC[i+1].R = 0x80A0;	//negative and turn on	
			}
		}else if(ministep==1&&direction==-1)
		{
			fullstep=3; //previous fullstep
			ministep = 0; //reset ministep count
			current_pos--;
			for(i=0;i<8;i=i+2)
			{
				SMC.DC[i].R = 0x80A0;	//turn off
				SMC.DC[i+1].R = 0x0000;	//negative and turn on	
			}
		}else
		{
			for(i=0;i<8;i=i+2)
			{
				SMC.DC[i].R -= direction*10;	//alter duty cycle based on direction
				SMC.DC[i+1].R += direction*10;	//alter duty cycle based on direction
			}
			ministep = (ministep + direction); //increment or decrement stepcount based on direction
			ministep = (ministep)%16; //keep step count between 0 and 15
			current_pos = (current_pos - direction);
		}
		break;
	case 1:
		if(ministep==15&&direction==1)
		{
			fullstep++; // next fullstep
			ministep = 0; // reset ministep count
			current_pos++;
			for(i=0;i<8;i=i+2)
			{
				SMC.DC[i].R = 0x00A0;	//turn off
				SMC.DC[i+1].R = 0x0000;	//negative and turn on	
			}
		}else if(ministep==1&&direction==-1)
		{
			fullstep--; //previous fullstep
			ministep = 0; //reset ministep count
			current_pos--;
			for(i=0;i<8;i=i+2)
			{
				SMC.DC[i].R = 0x8000;	//turn off
				SMC.DC[i+1].R = 0x80A0;	//negative and turn on	
			}
		}else
		{
			for(i=0;i<8;i=i+2)
			{
				SMC.DC[i].R += direction*10;	//alter duty cycle based on direction
				SMC.DC[i+1].R -= direction*10;	//alter duty cycle based on direction
			}
			ministep = (ministep + direction); //increment or decrement stepcount based on direction
			ministep = (ministep)%16; //keep step count between 0 and 15
			current_pos = (current_pos - direction);
		}
		break;
	case 2:
		if(ministep==15&&direction==1)
		{
			fullstep++; // next fullstep
			ministep = 0; // reset ministep count
			current_pos++;
			for(i=0;i<8;i=i+2)
			{
				SMC.DC[i].R = 0x8000;	//turn off
				SMC.DC[i+1].R = 0x00A0;	//negative and turn on	
			}
		}else if(ministep==1&&direction==-1)
		{
			fullstep--; //previous fullstep
			ministep = 0; //reset ministep count
			current_pos--;
			for(i=0;i<8;i=i+2)
			{
				SMC.DC[i].R = 0x00A0;	//turn off
				SMC.DC[i+1].R = 0x8000;	//negative and turn on		
			}
		}else
		{
			for(i=0;i<8;i=i+2)
			{
				SMC.DC[i].R -= direction*10;	//alter duty cycle based on direction
				SMC.DC[i+1].R += direction*10;	//alter duty cycle based on direction
			}
			ministep = (ministep + direction); //increment or decrement stepcount based on direction
			ministep = (ministep)%16; //keep step count between 0 and 15
			current_pos = (current_pos - direction);
		}
		break;
	case 3:
		if(ministep==15&&direction==1)
		{
			fullstep=0; // next fullstep
			ministep = 0; // reset ministep count
			current_pos++;
			for(i=0;i<8;i=i+2)
			{
				SMC.DC[i].R = 0x80A0;	//turn off
				SMC.DC[i+1].R = 0x8000;	//negative and turn on	
			}
		}else if(ministep==1&&direction==-1)
		{
			fullstep--; //previous fullstep
			ministep = 0; //reset ministep count
			current_pos--;
			for(i=0;i<8;i=i+2)  
			{
				SMC.DC[i].R = 0x0000;	//turn off
				SMC.DC[i+1].R = 0x00A0;	//negative and turn on	
			}
		}else
		{
			for(i=0;i<8;i=i+2)
			{
				SMC.DC[i].R += direction*10;	//alter duty cycle based on direction
				SMC.DC[i+1].R -= direction*10;	//alter duty cycle based on direction
			}
			ministep = (ministep + direction); //increment or decrement stepcount based on direction
			ministep = (ministep)%16; //keep step count between 0 and 15
			current_pos = (current_pos - direction);
		}
		break;
	}
	count++; /* increment tally of steps taken */
	if(count == 10000) /* reached end of cycle, change direction */
	{
		direction = direction * -1;
		count = 0;	/* reset count */
	}		
}

void DCU_init(uint32_t freq)
{   
    uint32_t i;
    CGM.AC0SC.R = 0x03000000; 	// Selects the FMPLL0 clock for pixel clock
    
//Sharp LQ043 set up
    LQ043_setup(freq);
    
    DCU.BGND.R = 0x00FF0000;	//Background colour = red
}


void LQ043_setup(uint32_t freq)
{
    uint32_t i;
// Config port A as output (red & green)
    for(i=PA0;i<PB0;i++) SIU.PCR[i].R = 0x0600;	//Enable output buffer
	
  // Config port PG0..7 as output (blue)
    for(i=PG0;i<PG8;i++) SIU.PCR[i].R = 0x0600;	//Enable output buffer

  // Config port PG8..11 as control
    SIU.PCR[PG8].R = 0x0600;  //Vsynch
    SIU.PCR[PG9].R = 0x0600; //Hsynch
    SIU.PCR[PG10].R = 0x0003; //DE just pull-up - connected to display ON
    SIU.PCR[PG11].R = 0x0600; //Pclk

    DCU.DSPSIZE.B.DELTAY = 272;	// Y = 272
    DCU.DSPSIZE.B.DELTAX = 30;	// X = 30 * 16 = 480
   
    DCU.HSYNPARA.B.BPH = 2;	// Horizontal back porch
    DCU.HSYNPARA.B.PWH = 41;	// Horizontal sync pulse
    DCU.HSYNPARA.B.FPH = 2;	// Horizontal front porch

    DCU.VSYNPARA.B.BPV = 2;	// Vertical back porch
    DCU.VSYNPARA.B.PWV = 10;	// Vertical sync pulse
    DCU.VSYNPARA.B.FPV = 2;	// Vertical front porch

    DCU.DIVRATIO.R = (uint32_t)(freq/8)-1;		//Clock divider
    DCU.SYNPOL.R = 0x00000003; // Synchro neg.
}
